/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Admin.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.admin;

import javax.management.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
* Admin
* @author Georges Goebel, Marc Calvisi
*/
public class Admin implements AdminMBean {
    private static MBeanServer mbs = null;
    private ObjectName m_relationObjectName = null;
    private ObjectName pumpName = null;

    private static Logger logger = Logger.getLogger(Admin.class.getName());

    public Admin() {

        ArrayList aList = MBeanServerFactory.findMBeanServer(null);
        //TODO see when mor MBeanServers are running
        for (Iterator it = aList.iterator(); it.hasNext();) {
            mbs = (MBeanServer) it.next();
        }

    }


    public void registerProbeInPump(String pump, String probe, Collection col, Long sampleTime) throws Exception {
        //System.out.println("registerProbeInPump");
        logger.log(Level.INFO, "registerProbeInPump " + probe);

        if (mbs.isRegistered(ObjectName.getInstance(probe))) {
            mbs.invoke(pumpName, "runMonitoringOnProbe", new Object[]{probe, col, sampleTime}, new String[]{"java.lang.String", "java.util.Collection", "java.lang.Long"});
        } else {
            //System.out.println("MBean Probe : "+probe+" is NOT registered in the MBeanServer");
            logger.log(Level.SEVERE, "MBean Probe : " + probe + " is NOT registered in the MBeanServer");
        }

    }

    public void createPump(String className, Long sampleRate) throws JMException {
        logger.log(Level.FINEST, "Create Pump");
        if (pumpName == null) {
            pumpName = new ObjectName(className);
            if (mbs.isRegistered(pumpName)) {
                logger.log(Level.FINER, "Pump is registered");
                if (sampleRate != null) {
                    logger.log(Level.FINER, "Setting the sample time of the pump to "+sampleRate);
                    mbs.setAttribute(pumpName, new Attribute("SampleTime", sampleRate));
                }


            } else {
                logger.log(Level.FINER, "Pump is NOT registered");
            }
        }
    }

    public Boolean modifyPumpCompression(Boolean state) throws ReflectionException, InstanceNotFoundException, MBeanException {
        logger.log(Level.FINEST, "Set compression for pump : " + state);
        if ((pumpName != null) && state != null) {
            mbs.invoke(pumpName, "modifyCompressing", new Object[]{state}, new String []{"java.lang.Boolean"});
        }
        return state;
    }


    public static MBeanServer getMBeanServer() {
        return mbs;
    }

}
