/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: ClientMx4j.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.client;

import javax.management.*;
import javax.management.monitor.GaugeMonitor;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Hashtable;


/**
* Client
* @author Georges Goebel, Marc Calvisi
*/
public class ClientMx4j {

    private static int counter = 0;

    public synchronized static void inc() {
        counter++;
        System.out.println("counter = " + counter);
    }

    public static void main(String[] args) {
        if ((args.length % 2) != 0) return;
        int i = 0;
        while (i < args.length) {
            Thread t = new Thread(new TT(args[i++], Integer.parseInt(args[i++])));
            t.start();
        }

        try {
            Thread thread = Thread.currentThread();
            synchronized (thread) {
                thread.wait();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }


    }

    private static class TT implements Runnable {

        private String host = null;
        private int port;

        GaugeMonitor gMon = new GaugeMonitor();

        public TT(String host, int port) {
            this.host = host;
            this.port = port;
        }


        public void run() {
            JMXServiceURL address;
            try {
                //              service:jmx:rmi://localhost/jndi/rmi://localhost:1090/jmx
                address = new JMXServiceURL("service:jmx:rmi://" + host + "/jndi/rmi://" + host + ":" + port + "/jmx");
                Hashtable environment = new Hashtable();

                /* IIOP  For info only. Not used
              String[] h=new String[4];
        h[0]="-ORBInitialPort";
        h[1]="2001";
        h[2]="-ORBInitialHost";
        h[3]="localhost";

          ORB orb = ORB.init(h, null);

          environment.put("java.naming.corba.orb", orb);

          address = new JMXServiceURL("service:jmx:iiop:///jndi/iiop://freci017015:2001/iiopconnector_jonas");
                */

                System.out.println("Address is : " + address);

                JMXConnector connector = JMXConnectorFactory.connect(address, environment);
                MBeanServerConnection cntion = connector.getMBeanServerConnection();
                System.out.println("connection : " + cntion);
                System.out.println("MBeans : " + cntion.getMBeanCount().intValue());
                ObjectName delegateName = ObjectName.getInstance("jimys:mbean=cpu");
                System.out.println("MBean : " + delegateName);

                NotificationListener listener = new NotificationListener() {

                    public void handleNotification(Notification arg0, Object arg1) {
                        System.out.println(arg0.getUserData());
                        
                        inc();
                    }
                };


                delegateName = ObjectName.getInstance("jimys:mbean=org.objectweb.jimys.monitoredSystem.pump");
                System.out.println("MBean : " + delegateName);
                cntion.addNotificationListener(delegateName, listener, null, null);

             //   System.out.println("Removing listener ...");

           //     cntion.removeNotificationListener(delegateName, listener);

            } catch (MalformedURLException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (MalformedObjectNameException e) {
                e.printStackTrace();
            }
            catch (InstanceNotFoundException e) {
                e.printStackTrace();
            }

        }

    }


    /**
     * Just for demonstation purpose how to implement a filter.
     * You can use it in this class
     */
    static class MyFilter implements NotificationFilter {

        public boolean isNotificationEnabled(Notification notification) {
            System.out.println("Filtering ... " + notification.getUserData());
            return notification.getUserData().toString().indexOf("HeapMemoryUsage") < 0;
        }
    }

}
