/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Cartography.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.cartography;

import org.objectweb.lewys.cartography.linux.*;
import org.objectweb.lewys.repository.cim.CIM_ManagedSystemElement;
import org.objectweb.jimys.monitoredSystem.pump.Pump;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
* Cartography
* @author Georges Goebel, Marc Calvisi
*/
public class Cartography implements CartographyMBean {

    private static final String ID = "Cartography";

    private Collection probes = new ArrayList();

    private static Logger logger = Logger.getLogger(Cartography.class.getName());
               /*
    private Probe_Controller controller = null;
    private Probe_DiskDrive diskDrive = null;
    private Probe_DiskPartition diskPartition = null;
    private Probe_IDEController ideController = null;
    private Probe_IPPort ipPort = null;
    private Probe_MediaAccessDevice mediaAccessDevice = null;
    private Probe_Memory memory = null;
    private Probe_MyrinetPort myrinetPort = null;
    private Probe_NetworkController networkController = null;
    private Probe_OperatingSystem operatingSystem = null;
    private Probe_PCIController pciController = null;
    private Probe_Processor processor = null;
    private Probe_RPMPackage rpmPackage = null;
    private Probe_SCIPort sciPort = null;
    private Probe_SCSIController scsiController = null;
    private Probe_SerialController serialController = null;
    private Probe_USBController usbController = null;
    private Probe_UnitaryComputerSystem unitaryComputerSystem = null;
    private Probe_VideoController videoController = null;
                 */

    public Cartography() {
        /*
        controller = new Probe_Controller();
        diskDrive = new Probe_DiskDrive();
        diskPartition = new Probe_DiskPartition();
        ideController = new Probe_IDEController();
        ipPort = new Probe_IPPort();
        mediaAccessDevice = new Probe_MediaAccessDevice();
        memory = new Probe_Memory();
        myrinetPort = new Probe_MyrinetPort();
        networkController = new Probe_NetworkController();
        operatingSystem = new Probe_OperatingSystem();
        pciController = new Probe_PCIController();
        processor = new Probe_Processor();
        rpmPackage = new Probe_RPMPackage();
        sciPort = new Probe_SCIPort();
        scsiController = new Probe_SCSIController();
        serialController = new Probe_SerialController();
        usbController = new Probe_USBController();
        unitaryComputerSystem = new Probe_UnitaryComputerSystem();
        videoController = new Probe_VideoController();
        */
        probes.add(new Probe_Controller());
        probes.add(new Probe_DiskDrive());
        probes.add(new Probe_DiskPartition());
        probes.add(new Probe_IDEController());
        probes.add(new Probe_IPPort());
        probes.add(new Probe_MediaAccessDevice());
        probes.add(new Probe_Memory());
        probes.add(new Probe_MyrinetPort());
        probes.add(new Probe_NetworkController());
        probes.add(new Probe_OperatingSystem());
        probes.add(new Probe_PCIController());
        probes.add(new Probe_Processor());
        //probes.add(new Probe_RPMPackage());
        probes.add(new Probe_SCIPort());
        probes.add(new Probe_SCSIController());
        probes.add(new Probe_SerialController());
        probes.add(new Probe_USBController());
        probes.add(new Probe_UnitaryComputerSystem());
        probes.add(new Probe_VideoController());

    }

    public String getIdentifier() {
        return ID;
    }

    /**
     * Difficult to implement : Because the probe is composed of different other probes which work
     * on certain system, it is difficult to say when the probe does not work. If we check all the underlying
     * probes, that consumes too much time and even then the evaluation is difficult. What do we say if the system
     * works but the probe is not able to parse the /proc filesystem ?
     * @return always true
     */
    public boolean isAlive() {
        return true;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public synchronized String getData () throws TransformerException {
        logger.log(Level.FINEST, "Getting Carthography data");
        Document doc = createDomDocument();
        Element cartography = doc.createElement("Cartography");

        for (Iterator i = probes.iterator(); i.hasNext();){
            Probe_ManagedSystemElement probe = (Probe_ManagedSystemElement) i.next();
            CIM_ManagedSystemElement[] systemElement = null;
            try {
                systemElement = probe.get();
            } catch (Exception e){
                //System.out.println("Error getting data in cartography");
                logger.log(Level.WARNING, "Error on cartography probe "+ probe);
                systemElement = null;
            }

            if (systemElement != null){
                Element elem = doc.createElement("Probe");
                elem.setAttribute("probeClass", probe.getClass().getName());

                for (int j = 0; j < systemElement.length; j++) {
                    CIM_ManagedSystemElement cim_managedSystemElement = systemElement[j];

                    StringTokenizer st = new StringTokenizer(cim_managedSystemElement.toString(), "\n");
                    while(st.hasMoreTokens()){
                        String line = st.nextToken();
                        if (line.indexOf(":") >= 0){
                            String name = line.substring(0, line.indexOf(":"));
                            String value = "";
                            if (line.indexOf(":")+2 <= line.length()){
                                value = line.substring(line.indexOf(":")+2, line.length());
                            }


                            Element attri = doc.createElement("ATTRIBUTE");
                            attri.setAttribute("name", name);
                            Node valueNode = doc.createTextNode(value);
                            attri.appendChild(valueNode);
                            elem.appendChild(attri);
                        }
                    }
                }
                cartography.appendChild(elem);
            }

        }
        doc.appendChild(cartography);

        return convertToString(doc);
    }

    private static Document createDomDocument() {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            return doc;
        } catch (ParserConfigurationException e) {
            logger.log(Level.SEVERE, "Error : createDomDocument :"+e);
        }
        return null;
    }

    private String convertToString(Document doc) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();

        DOMSource source = new DOMSource(doc);
        StringWriter sw = new StringWriter();
        StreamResult result = new StreamResult(sw);
        transformer.transform(source, result);
        String xmlString = sw.toString();
        return xmlString;
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }
    
}
