/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Cjdbc.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.cjdbc;

import org.objectweb.cjdbc.common.jmx.JmxConstants;
import org.objectweb.lewys.common.exceptions.NoSuchResourceException;
import org.objectweb.lewys.common.exceptions.ProbeException;
import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.objectweb.lewys.probe.jmx.cjdbc.CjdbcProbe;
import org.objectweb.jimys.monitoredSystem.pump.Pump;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringWriter;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
* Cjdbc probe
* @author Georges Goebel, Marc Calvisi
*/
public class Cjdbc implements CjdbcMBean {

    private static final String ID = "Cjdbc";

    private static Logger logger = Logger.getLogger(Cjdbc.class.getName());

    private CjdbcProbe cjdbcProbe = null;
    private int[] ressourceIDs = null;
    private String[] ressourceNames = null;

    /**
     * Needed for mlet to integrate jar in codebase !!!
     */
    private JmxConstants nullObject = null;


    public Cjdbc() {
        logger.log(Level.FINEST, "Creating CJDBC Probe");
        try {
            cjdbcProbe = new CjdbcProbe();
            ressourceIDs = cjdbcProbe.getResourceIds();
            ressourceNames = cjdbcProbe.getResourceNames();

        } catch (NoResourceToProbeException e) {
            logger.log(Level.SEVERE, "ERROR WHILE INITIALIZING THE CJDBC PROBE !!! \n"+e.getMessage());
        } catch (Exception e) {
            logger.log(Level.SEVERE, "ERROR WHILE INITIALIZING THE CJDBC PROBE !!! \n"+e.getMessage());
        }

    }



    public String getData() {
        logger.log(Level.FINEST, "Get Data from Cjdbc probe");
        String result = null;

        if (cjdbcProbe == null) {
            logger.log(Level.WARNING, "CJDBC Probe not properly initialized");
            return null;
        }
        try {
            long[] values = getValues();

            Document doc = createDomDocument();
            Element cjdbcElement = doc.createElement("CJDBCProbe");

            for (int i = 0; i < values.length; i++){
                Element res = doc.createElement("ATTRIBUTE");
                res.setAttribute("name", ressourceNames[i]);
                Node textNode = doc.createTextNode(Long.toString(values[i]));
                res.appendChild(textNode);
                cjdbcElement.appendChild(res);
            }
            doc.appendChild(cjdbcElement);
            result = convertToString(doc);

        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "ERROR IN CJDBC PROBE "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "ERROR IN CJDBC PROBE "+e);
        } catch (TransformerException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "ERROR IN CJDBC PROBE "+e);
        }
        return result;
    }

    public static Document createDomDocument() {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            return doc;
        } catch (ParserConfigurationException e) {
            logger.log(Level.SEVERE, "ERROR IN CJDBC PROBE : createDomDocument :"+e);
        }
        return null;
    }

    private long[] getValues() throws ProbeException, NoSuchResourceException {
        if (cjdbcProbe != null) {
            return cjdbcProbe.getValue(ressourceIDs);
        }
        return null;
    }

    private String convertToString(Document doc) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();

        DOMSource source = new DOMSource(doc);
        StringWriter sw = new StringWriter();
        StreamResult result = new StreamResult(sw);
        transformer.transform(source, result);
        String xmlString = sw.toString();
        return xmlString;
    }

    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        boolean result = false;
        try {
            Object values = getValues();
            if (values != null) {
                result = true;
            }
        } catch (ProbeException e) {
//            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.WARNING, "Warning : cjdbc probe not responding : ProbeException");
        } catch (NoSuchResourceException e) {
//            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.WARNING, "Warning : cjdbc probe not responding : NoSuchResourceException");
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

}
