/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Cpu.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.cpu;

import org.objectweb.jimys.monitoredSystem.pump.Pump;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
* Cpu probe
* @author Georges Goebel, Marc Calvisi
*/
public class Cpu implements CpuMBean {

    private static final String ID = "Cpu";

    private static Logger logger = Logger.getLogger(Cpu.class.getName());

    private float cpu = 0;

    private int cpuSampleTime = 3000;

    public Cpu() {
    }


    public Float getCPULoad() {
        //System.out.println("Accessing CPULoad methode");
        logger.log(Level.FINEST, "Getting CPULoad "+cpu);

        LinuxCpuStat stat = new LinuxCpuStat();
        synchronized (stat) {
            try {
                stat.wait(cpuSampleTime);
                stat.notify();
            } catch (InterruptedException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while getting cpu load "+e);
            }
        }
        LinuxCpuStatValues end = stat.stop();
        //return Float.toString(end.getCpuUsage());
        cpu = end.getCpuUsage();
        return new Float(cpu);

    }


    public float getCpu() {
        return this.cpu;
    }

    public int getCpuSampleTime() {
        return cpuSampleTime;
    }

    public void setCpuSampleTime(int cPUSampleTime) {
        this.cpuSampleTime = cPUSampleTime;
    }


    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        boolean result = false;
        try {
            LinuxCpuStat stat = new LinuxCpuStat();
            LinuxCpuStatValues c = stat.stop();
            c.getCpuUsage();
            result = true;
        } catch (Exception e){
            logger.log(Level.WARNING, "Warning CPU Probe not alive "+e);
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

}
