/*
 * Copyright (C) 2002-2003, Simon Nieuviarts
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.objectweb.jimys.monitoredSystem.probe.cpu;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;

public class LinuxCpuStat {
    LinuxCpuStatValues start = new LinuxCpuStatValues();
    LinuxCpuStatValues end = new LinuxCpuStatValues();

    public LinuxCpuStat() {
        readCounters(start);
    }

    public static void readCounters(LinuxCpuStatValues v) {
        try {
            FileReader fr = new FileReader("/proc/stat");
            BufferedReader in = new BufferedReader(fr);
            String line;
            while ((line = in.readLine()) != null) {
                StringTokenizer st = new StringTokenizer(line);
                if (!st.hasMoreTokens()) continue;
                String id = st.nextToken();
                if (id.equals("cpu")) {
                    v.user = Long.parseLong(st.nextToken());
                    v.nice = Long.parseLong(st.nextToken());
                    v.system = Long.parseLong(st.nextToken());
                    v.idle = Long.parseLong(st.nextToken());
                    return;
                }
            }
        } catch (IOException e) {
            // Nothing particular to do
        }
        v.user = 0;
        v.nice = 0;
        v.system = 0;
        v.idle = 0;
    }

    public void reset() {
        readCounters(start);
    }

    public LinuxCpuStatValues stop() {
        readCounters(end);
        end.diffWith(start);
        return end;
    }
}

