/*
 * Copyright (C) 2002-2003, Simon Nieuviarts
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.objectweb.jimys.monitoredSystem.probe.cpu;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.Serializable;

public class LinuxCpuStatValues implements Serializable {
    long user;
    long nice;
    long system;
    long idle;

    public void diffWith(LinuxCpuStatValues start) {
        user = user - start.user;
        nice = nice - start.nice;
        system = system - start.system;
        idle = idle - start.idle;
    }

    public long getCpuUsed() {
        return user + nice + system;
    }

    public long getCpuIdle() {
        return idle;
    }

    public float getCpuUsage() {
        long usage = user + nice + system;
        if ((idle + usage) == 0) return 100.0f;
        return ((float)((10000L * usage) / (idle + usage))) / 100.0f;
    }

    public void write(DataOutput out) throws IOException {
        out.writeLong(user);
        out.writeLong(nice);
        out.writeLong(system);
        out.writeLong(idle);
    }

    public void read(DataInput in) throws IOException {
        user = in.readLong();
        nice = in.readLong();
        system = in.readLong();
        idle = in.readLong();
    }

    public void copy(LinuxCpuStatValues v) {
        user = v.user;
        nice = v.nice;
        system = v.system;
        idle = v.idle;
    }

    public LinuxCpuStatValues() {
    }

    public LinuxCpuStatValues(LinuxCpuStatValues orig) {
        copy(orig);
    }
}

