/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: FileSystem.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.fileSystem;

import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.objectweb.lewys.common.exceptions.NoSuchResourceException;
import org.objectweb.lewys.common.exceptions.ProbeException;
import org.objectweb.lewys.probe.Probe;
import org.objectweb.lewys.probe.linux.DiskProbe;
import org.objectweb.jimys.monitoredSystem.pump.Pump;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringWriter;
import java.util.Vector;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
* FileSystem probe
* @author Georges Goebel, Marc Calvisi
*/
public class FileSystem implements FileSystemMBean{

    private static final String ID = "FileSystem";

    private static Logger logger = Logger.getLogger(FileSystem.class.getName());

    private Probe probe = null;
    private int[] ressourceid = null;

    private int sampleTime = 2000;


    public FileSystem() {
        try {
            probe = new DiskProbe();
            ressourceid = probe.getResourceIds();
        } catch (NoResourceToProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while creating FileSystem "+e);
        }

    }

    public String getData() {
        String result = "";
        if (probe == null) return null;
        try {
            long[] values = probe.getValue(ressourceid);

            for (int i = 0; i < values.length; i++) {
                long value = values[i];
                if (((i+1) % 11) == 0){
                    result += value + "\n";
                } else {
                    result += value + " ; ";
                }

            }
            return result;
        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while retieving data from FileSystem "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while retieving data from FileSystem "+e);
        }
        return null;
    }


    public String getTotalIOLoad () {
        if (probe == null) return null;
        String res = null;
        synchronized(this){
            Document doc = createDomDocument();
            Element fsElement = doc.createElement("FileSystemProbe");

            Vector col = new Vector();

            try {
                long[] values = probe.getValue(ressourceid);
                long timeStamp1 = System.currentTimeMillis();
                int nbrDevices = values.length / 11;
                int iovalue = 10;

                for (int i = 0; i < nbrDevices; i++) {
                    long value = values[i*11+iovalue];
                    col.add(new Long(value));
                }

                this.wait(sampleTime);

                values = probe.getValue(ressourceid);
                long timeStamp2 = System.currentTimeMillis();
                nbrDevices = values.length / 11;
                iovalue = 10;

                for (int i = 0; i < nbrDevices; i++) {
                    long value = values[i*11+iovalue];

                    Long oldValue = (Long) col.elementAt(i);

                    long divTime = timeStamp2 - timeStamp1;
                    long divValue = value - oldValue.longValue();


                    Double result = new Double(((double)divValue / (double)divTime) * 100);
                    col.setElementAt(probe.getResourceName(i*11+iovalue) + " LOAD = "+result, i);

                    Element att = doc.createElement("ATTRIBUTE");
                    att.setAttribute("name", probe.getResourceName(i*11+iovalue) + " LOAD");
                    Node valueText = doc.createTextNode(result.toString());
                    att.appendChild(valueText);
                    fsElement.appendChild(att);

                }

                doc.appendChild(fsElement);
                res = convertToString(doc);

                return res;

            } catch (ProbeException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while retieving data from FileSystem "+e);
            } catch (NoSuchResourceException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while retieving data from FileSystem "+e);
            } catch (InterruptedException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while retieving data from FileSystem "+e);
            } catch (TransformerException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while retieving data from FileSystem "+e);
            }
            return res;

        }
   //     return new ArrayList();
    }

    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        if (getData() != null){
            return true;
        }
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    private String convertToString(Document doc) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();

        DOMSource source = new DOMSource(doc);
        StringWriter sw = new StringWriter();
        StreamResult result = new StreamResult(sw);
        transformer.transform(source, result);
        String xmlString = sw.toString();
        return xmlString;
    }

   public static Document createDomDocument() {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            return doc;
        } catch (ParserConfigurationException e) {
            logger.log(Level.SEVERE, "Error while createDomDocument in FileSystem "+e);
        }
        return null;
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }
    
}
