/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: GenericJmx.java,v 1.4 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.genericJmx;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.objectweb.jimys.monitoredSystem.pump.Pump;

import javax.management.*;
import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.CompositeType;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;
import java.util.*;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
* GenericJmx probe
* @author Georges Goebel, Marc Calvisi
*/
public class GenericJmx implements GenericJmxMBean {
    private String ID = "GenericJmx";

    private static Logger logger = Logger.getLogger(GenericJmx.class.getName());

    private MBeanServerConnection mbsc = null;

    private Hashtable monitoredAttributes = new Hashtable();
    protected String rootNodeName = "GenericJmx";
    protected String configFile = null;
    protected String jmxUrl = null;

    public void init(String jmxUrl, String configFile, String rootNodeName) {
        if (rootNodeName != null) {
            this.rootNodeName = rootNodeName;
            this.ID = rootNodeName;
        }
        try {
            this.configFile = configFile;
            Document doc = parseXml(configFile);

            try {
                // Get the matching elements
                NodeList nodelist = org.apache.xpath.XPathAPI.selectNodeList(doc, "/probe-config/polls-descriptor/poll");

                // Process the elements in the nodelist
                for (int i = 0; i < nodelist.getLength(); i++) {
                    // Get element
                    Element elem = (Element) nodelist.item(i);

                    NodeList nl = elem.getElementsByTagName("discriminator");
                    //System.out.println("nl.getLength() = " + nl.getLength());
                    String discrimator = null;
                    if (nl.getLength() == 1) {
                        Node nDiscrimator = nl.item(0);
                        discrimator = nDiscrimator.getFirstChild().getNodeValue();
                    }

                    NodeList aNames = elem.getElementsByTagName("attribute-name");
                    Collection cAttributes = new ArrayList();
                    for (int j = 0; j < aNames.getLength(); j++) {
                        Node nAtt = aNames.item(j);
                        String attribute = nAtt.getFirstChild().getNodeValue();
                        cAttributes.add(attribute);
                    }

                    monitoredAttributes.put(discrimator, cAttributes);
                }
            } catch (javax.xml.transform.TransformerException e) {
                //e.printStackTrace();
                logger.log(Level.SEVERE, "Error while initializing (parsing xml config) genericjmx probe : "+jmxUrl+"\n"+e);
            }


            JMXServiceURL address = null;
            this.jmxUrl = jmxUrl;
            address = new JMXServiceURL(jmxUrl);
            Map environment = new HashMap();
            JMXConnector connector = JMXConnectorFactory.connect(address, environment);
            connector.connect();
            mbsc = connector.getMBeanServerConnection();

        } catch (Exception e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while initializing genericjmx probe : "+jmxUrl+"\n"+e);
        }
    }

    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        if (mbsc != null) {
            Integer count = null;
            try {
                count = mbsc.getMBeanCount();
            } catch (IOException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.WARNING, "GenericJmx is not alive "+ID);
                // try reconnect
                if ((jmxUrl != null) && (configFile != null)) {
                    logger.log(Level.INFO, " Try to Reconnect ");
                    this.init(jmxUrl,configFile,rootNodeName);
                }

            }
            if (count != null) return true;
        }
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public synchronized String getData() {

        if (mbsc != null) {
            Document doc = createDomDocument();
            Element rootElement = doc.createElement(rootNodeName);
            doc.appendChild(rootElement);

            Enumeration enumeration = monitoredAttributes.keys();
            while (enumeration.hasMoreElements()) {
                String s = (String) enumeration.nextElement();

                try {
                    ObjectName oName = ObjectName.getInstance(s);
                    Set mBeans = mbsc.queryNames(oName, null);

                    Collection col = (Collection) monitoredAttributes.get(s);
                    for (Iterator iterator = mBeans.iterator(); iterator.hasNext();) {
                        ObjectName objectName = (ObjectName) iterator.next();

                        Element mbean = doc.createElement("mbean");
                        mbean.setAttribute("name", objectName.toString());
                        mbean.setAttribute("time", Long.toString(new Date().getTime()));

                        for (Iterator iterator1 = col.iterator(); iterator1.hasNext();) {
                            String attrib = (String) iterator1.next();
                            Object o = null;
                            try {
                                o = mbsc.getAttribute(objectName, attrib);

                            } catch (AttributeNotFoundException e) {
                                o = "notFound";
                                // e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                            }

                            if (o instanceof javax.management.openmbean.CompositeDataSupport) {
                                CompositeDataSupport cds = (CompositeDataSupport) o;
                                CompositeType ct = cds.getCompositeType();
                                Set kSet = ct.keySet();
                                for (Iterator iterator2 = kSet.iterator(); iterator2.hasNext();) {
                                    Object o1 = iterator2.next();
                                    Object oValue = cds.get(o1.toString());

                                    Element at = doc.createElement("ATTRIBUTE");
                                    at.setAttribute("name", o1.toString());
                                    Text textNode = doc.createTextNode(oValue.toString());
                                    at.appendChild(textNode);
                                    mbean.appendChild(at);
                                }

                            } else {
                                Element attribu = doc.createElement("ATTRIBUTE");
                                attribu.setAttribute("name", attrib);
                                Node node = doc.createTextNode(o.toString());
                                attribu.appendChild(node);

                                mbean.appendChild(attribu);

                            }

                        }
                        rootElement.appendChild(mbean);
                    }


                } catch (MalformedObjectNameException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error in GenericJmx Probe MalformedObjectNameException "+e);
                } catch (IOException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error in GenericJmx Probe IOException "+e);
                } catch (ReflectionException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error in GenericJmx Probe ReflectionException "+e);
                } catch (InstanceNotFoundException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error in GenericJmx Probe InstanceNotFoundException "+e);
                } catch (MBeanException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error in GenericJmx Probe MBeanException "+e);
                }
            }

            try {
                return convertToString(doc);
            } catch (TransformerException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error in GenericJmx Probe "+e);
            }

        } else {
            logger.log(Level.WARNING,"You must connect to a mbeanserver");
        }

        return null;

    }

    public synchronized static Document parseXml(String s) {
        DOMParser d = new DOMParser();
        try {
            FileReader sr = new FileReader(s);
            InputSource is = new InputSource(sr);
            d.parse(is);
            Document doc = d.getDocument();
            sr.close();
            return doc;
        } catch (Exception e) {
            //e.printStackTrace();
            logger.log(Level.SEVERE, "Error in GenericJmx Probe : parseXml : "+e);
        }
        return null;
    }

    private static Document createDomDocument() {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            return doc;
        } catch (ParserConfigurationException e) {
            logger.log(Level.SEVERE, "Error in GenericJmx Probe : createDomDocument : "+e);
        }
        return null;
    }

    private String convertToString(Document doc) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();

        DOMSource source = new DOMSource(doc);
        StringWriter sw = new StringWriter();
        StreamResult result = new StreamResult(sw);
        transformer.transform(source, result);
        String xmlString = sw.toString();
        return xmlString;
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getID() {
        return ID;
    }

    public void setID(String ID) {
        this.ID = ID;
    }

    public MBeanServerConnection getMbsc() {
        return mbsc;
    }

    public void setMbsc(MBeanServerConnection mbsc) {
        this.mbsc = mbsc;
    }

    public Hashtable getMonitoredAttributes() {
        return monitoredAttributes;
    }

    public void setMonitoredAttributes(Hashtable monitoredAttributes) {
        this.monitoredAttributes = monitoredAttributes;
    }

    public String getRootNodeName() {
        return rootNodeName;
    }

    public void setRootNodeName(String rootNodeName) {
        this.rootNodeName = rootNodeName;
    }
}
