/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: GenericLogFile.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.genericLogFile;

import org.objectweb.jimys.monitoredSystem.pump.Pump;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
* GenericLogFile probe
* @author Georges Goebel, Marc Calvisi
*/
public class GenericLogFile implements GenericLogFileMBean {

    private String ID = "GenericLogFile";

    private static Logger logger = Logger.getLogger(GenericLogFile.class.getName());

    private String filePath = null;

    private DateFormat myDateFormat = null;
    private Date currentErrorDate = null;

    private String errorLevel = "[error]";
    private String strFormat = "EEE MMM dd hh:mm:ss yyyy";


    public void init(String filePath) {
        this.filePath = filePath;
        myDateFormat = new SimpleDateFormat(strFormat);
        currentErrorDate = new Date();
    }

    public void init(String filePath, String errorLevel, String timeFormat) {
        this.filePath = filePath;
        this.strFormat = timeFormat;
        this.errorLevel = errorLevel;
        myDateFormat = new SimpleDateFormat(timeFormat);
        currentErrorDate = new Date();
    }

    public void init(String filePath, String errorLevel, String timeFormat, String id) {
        this.filePath = filePath;
        this.strFormat = timeFormat;
        this.errorLevel = errorLevel;
        if (id != null) {
            this.ID = id;
        }
        myDateFormat = new SimpleDateFormat(timeFormat);
        currentErrorDate = new Date();
    }


    public String getLog() {
        logger.log(Level.FINEST, "Getting GenericLog");
        if (filePath == null) {
            logger.log(Level.SEVERE, "GenericLog Probe not initialized");
            return null;
        }
        String result = "";
        try {
            FileReader fr = new FileReader(filePath);
            BufferedReader in = new BufferedReader(fr);
            String line;
            while ((line = in.readLine()) != null) {
                int index = line.indexOf(errorLevel);
                if (index > -1) {
                    String date = line.substring(0, index);
                    date = date.trim();
                    date = date.substring(1, date.length() - 1);
                    Date tempDate = null;
                    try {
                        tempDate = myDateFormat.parse(date);
                    } catch (ParseException e) {
                        //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    }

                    if ((tempDate != null) && (currentErrorDate.getTime() < tempDate.getTime())) {
                        currentErrorDate = tempDate;
                        result += line + '\n';
                    }
                }
            }
            in.close();
            return result;
        } catch (IOException e) {
            //e.printStackTrace();
            logger.log(Level.SEVERE, "ERROR IN GENERICLOGPROBE : " + e);
        }

        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }


    public String getFilePath() {
        return filePath;
    }

    public void setFilePath(String filePath) {
        this.filePath = filePath;
    }

    public String getErrorLevel() {
        return errorLevel;
    }

    public void setErrorLevel(String errorLevel) {
        this.errorLevel = errorLevel;
    }

    public String getDateFormat() {
        return strFormat;
    }

    public void setDateFormat(String strFormat) {
        this.strFormat = strFormat;
    }

    public String getIdentifier() {
        return ID;
    }

    public Long getFileSize() {
        if (filePath != null) {
            File f = new File(filePath);
            return new Long(f.length());
        }
        logger.log(Level.FINE, "No log filename specified (null)");
        return new Long(0);
    }

    public boolean isAlive() {
        if (filePath != null){
            return true;
        }
        logger.log(Level.WARNING, "No filename specified. Probe alive but ressource not available");
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getFullIdentifier() {
        return Pump.getHostName() + ";domain;instance;probe;" + ID;  //To change body of implemented methods use File | Settings | File Templates.
    }
}
