/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: GenericLogFileMBean.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.genericLogFile;

import org.objectweb.jimys.monitoredSystem.probe.Common;

/**
* GenericLogFile probe
* @author Georges Goebel, Marc Calvisi
*/
public interface GenericLogFileMBean extends Common{
    /**
     * Initialise the apache probe
     * @param filePath The path of the log file to monitor
     */
    public void init(String filePath);

    /**
     * Initialise the apache probe. The default value of the id is "GenericLogFile"
     * @param filePath The path of the log file to monitor
     * @param errorLevel The error level string you want to monitor
     * @param timeFormat The time format pattern used in the log file
     */
    public void init(String filePath, String errorLevel, String timeFormat);

    /**
     * Initialise the apache probe. The same as init(String filePath, String errorLevel, String timeFormat)
     * But it let you specify the id of the probe. If you use this probe more times in the system, you may
     * specify different ids for the different instances of this probe. This let you distigue between
     * the different results. If id = null the default value will be used "GenericLogFile"
     *
     * @param filePath The path of the log file to monitor
     * @param errorLevel The error level string you want to monitor
     * @param timeFormat The time format pattern used in the log file
     * @param id The id of the probe
     */
    public void init(String filePath, String errorLevel, String timeFormat, String id);

    /**
     * Get the apache error log
     * @return The apache error log
     */
    public String getLog ();

    /**
     * Getter for the apache error log file
     * @return the path to the logfile
     */
    public String getFilePath();

    /**
     * Setter for the apache error log file
     * @param filePath The new Path to the logfile
     */
    public void setFilePath(String filePath);

    /**
     * Getter for the error level to observe
     * @return The error level
     */
    public String getErrorLevel();

    /**
     * Setter for the serror level
     * @param errorLevel The new error level
     */
    public void setErrorLevel(String errorLevel);

    /**
     * Getter for the date format in the apache log file
     * @return The date format
     */
    public String getDateFormat();

    /**
     * Setter for the ate format
     * @param strFormat The new Date format to use
     */
    public void setDateFormat(String strFormat);

    /**
     * Returns the size of the monitored log file
     * @return File size
     */
    public Long getFileSize();

}
