/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Jonas.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.jonas;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.objectweb.jimys.monitoredSystem.pump.Pump;
import shell.ProbeShell;

import javax.management.*;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.IOException;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.util.*;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * GenericLogFile probe
 * @author Georges Goebel, Marc Calvisi
 * @deprecated Attention : Use jonasJmx probe -> this is only if you want to use the jonas probe
 * of Fran?ois
 */
public class Jonas implements JonasMBean {
    private static final String ID = "Jonas";

    private String serverURL = null;
    private MBeanServerConnection mbs = null;
    private ObjectName oName = null;
    private String attribute = null;

    private static Logger logger = Logger.getLogger(Jonas.class.getName());


    public String getIdentifier() {
        return ID;
    }

    public boolean isAlive() {
        boolean result = false;
        if (mbs != null){
            Integer count = null;
            try {
                count = mbs.getMBeanCount();
            } catch (IOException e) {
                logger.log(Level.WARNING, "Jonas Probe not alive ");
            }
            if (count != null) {
                result = true;
            }
        }
        return result;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public void init(String probeHome, String probeConfigProperties, String serverURL, String objectName, String attribute) {
        //String[] ar = new String[]{"-org.objectweb.jimys.monitoredSystem.start", "org.objectweb.jimys.monitoredSystem.probe", "-jonas", "-r", "/home/goebelg/IdeaProjects/prototype/conf/probe_default.properties"};
        String[] ar = new String[]{"-start", "probe", "-jonas", "-r", probeConfigProperties};
        System.setProperty("probe.home", probeHome);
        try {
            ProbeShell.shellUtil(ar);
            init(serverURL, objectName, attribute);
        } catch (Exception e){
            //e.printStackTrace();
            logger.log(Level.SEVERE, "Error initializing Jonas probe "+e);
        }


    }

    public void init(String serverURL, String objectName, String attribute) {
        this.serverURL = serverURL;
        this.attribute = attribute;


        JMXServiceURL address = null;
        try {
            //address = new JMXServiceURL("rmi", "localhost", 0, "/jndi/jrmp");
            address = new JMXServiceURL(serverURL);
            Map environment = new HashMap();
            //environment.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.rmi.registry.RegistryContextFactory");
            //environment.put(Context.PROVIDER_URL, "rmi://" + "localhost" + ":" + "1090");
            //environment.put(Context.PROVIDER_URL, serverURL);
            JMXConnector connector = JMXConnectorFactory.connect(address, environment);
            connector.connect();
            mbs = connector.getMBeanServerConnection();

            oName = ObjectName.getInstance(objectName);
        } catch (MalformedURLException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error initializing Jonas probe "+e);
        } catch (IOException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error initializing Jonas probe "+e);
        } catch (MalformedObjectNameException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error initializing Jonas probe "+e);
        }

    }


    public synchronized String getData() throws ReflectionException, InstanceNotFoundException, IOException, MBeanException, AttributeNotFoundException {
        if (mbs == null) {
            logger.log(Level.WARNING, "You must first init the MBean Jonas !");
            return null;
        }
        Collection col = (Collection) mbs.getAttribute(oName, attribute);
        String result = null;

        if (col != null) {
            Document doc = createDomDocument();
            Element rootElement = doc.createElement("Jonas");
            for (Iterator iterator = col.iterator(); iterator.hasNext();) {
                // one mbean result
                String s = (String) iterator.next();

                String headerLine = s.substring(0, s.indexOf("\n"));
                String data = s.substring(s.indexOf("\n") + 1, s.length());

                StringTokenizer st = new StringTokenizer(headerLine, ";");
                int length = st.countTokens();
                String[] attributeNames = new String[length];

                int count = 0;
                while (st.hasMoreTokens()) {
                    attributeNames[count++] = st.nextToken();
                }


                // analyse data
                st = new StringTokenizer(data, "\n");

                while (st.hasMoreTokens()) {
                    StringTokenizer dataSt = new StringTokenizer(st.nextToken(), ";");

                    Element mBeanElement = doc.createElement("mbean");

                    count = 0;
                    while (dataSt.hasMoreTokens()) {
                        String value = dataSt.nextToken();

                        switch (count) {
                            case 0:
                                {
                                    mBeanElement.setAttribute("name", value);
                                    break;
                                }

                            case 1:
                                {
                                    mBeanElement.setAttribute("date", value);
                                    break;
                                }

                            case 2:
                                {
                                    mBeanElement.setAttribute("time", value);
                                    break;
                                }
                            default :
                                {
                                    Element tempElement = doc.createElement("ATTRIBUTE");
                                    tempElement.setAttribute("name", attributeNames[count]);
                                    //tempElement.setAttribute("value", value);
                                    Node textNode = doc.createTextNode(value);
                                    tempElement.appendChild(textNode);

                                    mBeanElement.appendChild(tempElement);
                                }

                        }
                        count++;

                    }

                    rootElement.appendChild(mBeanElement);
                }
            }
            doc.appendChild(rootElement);
            try {
                result = convertToString(doc);
            } catch (TransformerException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error retrieving data from the Jonas probe "+e);
            }
        }


        return result;
    }

    private static Document createDomDocument() {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            return doc;
        } catch (ParserConfigurationException e) {
            logger.log(Level.SEVERE, "Error createDomDocument in the Jonas probe "+e);
        }
        return null;
    }

    private String convertToString(Document doc) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();

        DOMSource source = new DOMSource(doc);
        StringWriter sw = new StringWriter();
        StreamResult result = new StreamResult(sw);
        transformer.transform(source, result);
        String xmlString = sw.toString();
        return xmlString;
    }

    //TODO implement the domain and instance name
    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

}
