/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: JonasJmx.java,v 1.3 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.jonasJmx;

import org.objectweb.jimys.monitoredSystem.probe.genericJmx.GenericJmx;
import org.objectweb.jimys.monitoredSystem.pump.Pump;

import javax.management.*;
import java.io.IOException;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
* JonasJmx probe
* @author Georges Goebel, Marc Calvisi
*/
public class JonasJmx extends GenericJmx implements JonasJmxMBean {
    private String ID = "Jonas";
    private String FULLIDENTIFIER = null;

    private static Logger logger = Logger.getLogger(JonasJmx.class.getName());

    public JonasJmx() {
        this.setID(ID);
        this.setRootNodeName(ID);
    }


    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        if (getMbsc() != null) {
            Integer count = null;
            try {
                count = getMbsc().getMBeanCount();
            } catch (IOException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.WARNING, "JonasJmx is not alive "+ID);
                // try reconnect
                if ((jmxUrl != null) && (configFile != null)) {
                    logger.log(Level.INFO, " Try to Reconnect ");
                    this.init(jmxUrl,configFile,rootNodeName);
                }

            }
            if (count != null) return true;
        }
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }


    public String getFullIdentifier() {
        String instanceName = "instance";
        String domainName = "domain";


        if (FULLIDENTIFIER == null) {
            if (getMbsc() != null) {
                try {
                    ObjectName oGenericName = ObjectName.getInstance("*:j2eeType=J2EEServer,*");
                    Set set = getMbsc().queryNames(oGenericName, null);
                    ObjectName oName = (ObjectName) set.iterator().next();
                    if (oName != null) {
                        instanceName = (String)getMbsc().getAttribute(oName, "serverName");
                        domainName = oName.getDomain();
                    }
                } catch (MalformedObjectNameException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (ReflectionException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (IOException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (InstanceNotFoundException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (MBeanException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                } catch (AttributeNotFoundException e) {
                    e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                }

            }
            FULLIDENTIFIER = Pump.getHostName()+";"+domainName+";"+instanceName+";probe;"+ID;
        }
        if (FULLIDENTIFIER == null) {
            return Pump.getHostName()+";"+"domain"+";"+"instance"+";probe;"+ID;
        }
        return FULLIDENTIFIER;  //To change body of implemented methods use File | Settings | File Templates.
    }

}
