/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: SunJvm.java,v 1.3 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.jvm15;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;
import org.objectweb.jimys.monitoredSystem.pump.Pump;

import javax.management.*;
import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.CompositeType;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringWriter;
import java.util.*;
import java.util.logging.Logger;
import java.util.logging.Level;

/**
 * @author Georges Goebel, Marc Calvisi
 * @deprecated Dont use this probe any longer. Use the genericJmX Probe to monitor the jvm 1.5
 */
public class SunJvm implements SunJvmMBean {
    private static final String ID = "SunJvm";
    private MBeanServerConnection mbsc = null;

    private Properties props = null;

    private static Logger logger = Logger.getLogger(SunJvm.class.getName());

    public void init(String properties) {

        props = new Properties();
        try {
            props.load(new FileInputStream(new File(properties)));

            if (props.getProperty("jxm.url") != null) {
                JMXServiceURL u = null;
                u = new JMXServiceURL(props.getProperty("jxm.url"));
                JMXConnector c = JMXConnectorFactory.connect(u);
                mbsc = c.getMBeanServerConnection();
            }
        } catch (IOException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error initializing the sunjvm probe "+e);
            props = null;
        }

        //JMXServiceURL u = new JMXServiceURL("service:jmx:rmi:///jndi/rmi:// ? + hostName + ":" + portNum +  "/jmxrmi");

    }


    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        if (mbsc != null) {
            Integer i = null;
            try {
                i = mbsc.getMBeanCount();
            } catch (IOException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.WARNING, "Warning sunjvm probe not alive");
            }
            if (i != null) {
                return true;
            }
        }
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public synchronized String getJvmData() {

        String mBeansList = props.getProperty("jmx.mbeans");
        if (mBeansList != null) {
            Document doc = createDomDocument();
            Element fsElement = doc.createElement(ID);

            StringTokenizer st = new StringTokenizer(mBeansList, ";");
            int i = 1;
            while (st.hasMoreTokens()) {
                String mBean = st.nextToken();
                try {
                    ObjectName oName = ObjectName.getInstance(mBean);
                    String attributesList = props.getProperty("mBean" + i++ + ".attributes");
                    StringTokenizer st2 = new StringTokenizer(attributesList, ";");

                    Element el = doc.createElement("mbean");
                    el.setAttribute("date", new Long(new Date().getTime()).toString());
                    el.setAttribute("name", oName.toString());

                    while (st2.hasMoreTokens()) {
                        String attribute = st2.nextToken();
                        Object o = mbsc.getAttribute(oName, attribute);
                        if (o instanceof javax.management.openmbean.CompositeDataSupport) {
                            CompositeDataSupport cds = (CompositeDataSupport) o;
                            CompositeType ct = cds.getCompositeType();
                            Set kSet = ct.keySet();
                            for (Iterator iterator = kSet.iterator(); iterator.hasNext();) {
                                Object o1 = iterator.next();
                                Object oValue = cds.get(o1.toString());
                                System.out.println("key : " + o1 + "values = " + oValue);

                                Element at = doc.createElement("ATTRIBUTE");
                                at.setAttribute("name", o1.toString());
                                Text textNode = doc.createTextNode(oValue.toString());
                                at.appendChild(textNode);
                                el.appendChild(at);
                            }
                        } else {
                            Element at = doc.createElement("ATTRIBUTE");
                            at.setAttribute("name", attribute);
                            Text textNode = doc.createTextNode(o.toString());
                            at.appendChild(textNode);
                            el.appendChild(at);
                        }
                    }
                    fsElement.appendChild(el);
                } catch (MalformedObjectNameException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error sunjvm probe "+e);
                } catch (ReflectionException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error sunjvm probe "+e);
                } catch (IOException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error sunjvm probe "+e);
                } catch (InstanceNotFoundException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error sunjvm probe "+e);
                } catch (MBeanException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error sunjvm probe "+e);
                } catch (AttributeNotFoundException e) {
                    //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                    logger.log(Level.SEVERE, "Error sunjvm probe "+e);
                }

            }
            doc.appendChild(fsElement);
            try {
                return convertToString(doc);
            } catch (TransformerException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error sunjvm probe "+e);
            }
        }
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    private String convertToString(Document doc) throws TransformerException {
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();

        DOMSource source = new DOMSource(doc);
        StringWriter sw = new StringWriter();
        StreamResult result = new StreamResult(sw);
        transformer.transform(source, result);
        String xmlString = sw.toString();
        return xmlString;
    }

    public static Document createDomDocument() {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            Document doc = builder.newDocument();
            return doc;
        } catch (ParserConfigurationException e) {
            logger.log(Level.SEVERE, "Error sunjvm probe createDomDocument"+e);
        }
        return null;
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }
    
}
