/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Log.java,v 1.4 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.log;

import org.objectweb.jimys.monitoredSystem.pump.Pump;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import java.io.IOException;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
* Log probe
* @author Georges Goebel, Marc Calvisi
*/
public class Log implements LogMBean{

    private static final String ID = "JonasLog";


    MBeanServerConnection jonasServer = null;

    private static Logger logger = Logger.getLogger(Log.class.getName());

    public Log() {
    }

    //private Level level = Level.SEVERE;
    private Level level = Level.INFO;

    private String jmxURL = null;
    private String mBeanName = null;
    
    private String logData = "";


    public void init (String jmxURL, String mBeanName) throws Exception {
        this.jmxURL = jmxURL;
        this.mBeanName = mBeanName;
        JMXServiceURL jonasAddress = null;
        Hashtable environment = new Hashtable();

        jonasAddress = new JMXServiceURL(jmxURL);
        JMXConnector connection = JMXConnectorFactory.newJMXConnector(jonasAddress, environment);
        connection.connect(environment);
        jonasServer = connection.getMBeanServerConnection();

        ObjectName oName = ObjectName.getInstance(mBeanName);
        LogListener listener = new LogListener(this);
        //jonasServer.addNotificationListener(oName, listener, null, "monolog");
        jonasServer.addNotificationListener(oName, listener, null, null);
    }


    public void init (String jmxURL, String mBeanName, String sLevel) throws Exception {
        if (sLevel != null) {
            level = Level.parse(sLevel);
            if (level == null) {
                level = Level.INFO;
            }
        }
        this.jmxURL = jmxURL;
        this.mBeanName = mBeanName;

        JMXServiceURL jonasAddress = null;
        Hashtable environment = new Hashtable();

        jonasAddress = new JMXServiceURL(jmxURL);
        JMXConnector connection = JMXConnectorFactory.newJMXConnector(jonasAddress, environment);
        connection.connect(environment);
        jonasServer = connection.getMBeanServerConnection();

        ObjectName oName = ObjectName.getInstance(mBeanName);
        LogListener listener = new LogListener(this);
        jonasServer.addNotificationListener(oName, listener, null, "monolog");
    }


    /*
    public String getJonasErrorLog (){
        String result = "";
        if (logFile != null){
            try {
                FileReader fr = new FileReader(logFile);
                BufferedReader in = new BufferedReader(fr);
                String line;
                while ((line = in.readLine()) != null) {
                    int index = line.indexOf(logLevel);
                    if (index > -1){
                        String date = line.substring(0, index);
                        date = date.trim();
                        date = date.substring(0, date.indexOf(','));
                        Date tempDate = myDateFormat.parse(date);

                        //if (currentDate == null) currentDate = tempDate;
                        if (tempDate.getTime() > currentDate.getTime()){
                            currentDate = tempDate;
                            result += line + '\n';
                        }
                    }
                }
                in.close();
            } catch (IOException e) {
                //e.printStackTrace();
                logger.log(Level.SEVERE, "Error while getJonasErrorLog "+e);
            } catch (ParseException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while getJonasErrorLog "+e);
            }
        } else {
            logger.log(Level.WARNING,"YOU MUST INVOKE THE INIT METHODE BEFORE USING THE LOG PROBE");
        }
        if (result.equals("")) result = null;

        return result;
    }
    */

    public String getJonasErrorLog (){
        logger.log(Level.FINEST,"Getting log Data "+logData);
        if (logData.equals("")) return null;
        String result = logData;
        logData = "";
        return result;
    }

    public void setData (String data) {
        logger.log(Level.FINEST,"Setting log Data "+data);
        synchronized (logData) {
            logData += data + "\n";
        }
    }

    public Level getLevel() {
        return level;
    }

    public void setLevel(Level level) {
        this.level = level;
    }

    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        if (jonasServer != null) {
            Integer i = null;
            try {
                i = jonasServer.getMBeanCount();
            } catch (IOException e) {
               // e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.WARNING,"Log Probe not alive "+e);
                // try reconnect
                if ((jmxURL != null) && (mBeanName != null)) {
                    logger.log(Level.INFO, " Try to Reconnect ");
                    try {
                        this.init(jmxURL,mBeanName,level.toString());
                    } catch (Exception e1) {
                        logger.log(Level.WARNING,"Error reconnecting "+e1);
                    }
                    logger.log(Level.INFO, " Reconnect Successfull");
                }

            }
            if (i != null) {
                return true;
            } else {
                return false;
            }
        }

        return false;
    }
    
    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

}
