/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Mem.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.mem;

import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.objectweb.lewys.common.exceptions.NoSuchResourceException;
import org.objectweb.lewys.common.exceptions.ProbeException;
import org.objectweb.lewys.probe.Probe;
import org.objectweb.lewys.probe.linux.MemoryProbe;
import org.objectweb.jimys.monitoredSystem.pump.Pump;

import javax.management.timer.Timer;
import java.util.Hashtable;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
* Memory probe
* @author Georges Goebel, Marc Calvisi
*/
public class Mem implements MemMBean {
    private static final String ID = "Memory";

    private Probe memProbe = null;
    Hashtable notificationlisteners = new Hashtable();

    private Timer timer = new Timer();
    private Integer notificationId = null;
    private static final long PERIOD = 5000;//Timer.ONE_SECOND;

    private static Logger logger = Logger.getLogger(Mem.class.getName());

    public Mem() {
        logger.log(Level.FINEST,"Constucting Memory MBean !");
        try {
            memProbe = new MemoryProbe();
        } catch (NoResourceToProbeException e) {
            //e.printStackTrace();
            logger.log(Level.SEVERE,"Error creating memory probe "+e);
        }


    }

    /* (non-Javadoc)
     * @see org.objectweb.jimys.monitoredSystem.probe.mem.MemMBean#getMemory()
     */
    public String getMemory() {
        String result = "No Memory result";
        long freeMem = 0;
        long usedMem = 0;
        long totalMem = 0;

        if (memProbe == null) return null;
        
        try {
            long[] res = memProbe.getValue(memProbe.getResourceIds());

            result = "Total Memory : " + Long.toString(res[0]);
            result += "\nUsed Memory : " + Long.toString(res[1]);
            result += "\nFree Memory : " + Long.toString(res[2]);
            result += "\nSwap total : " + Long.toString(res[6]);
            result += "\nSwap used : " + Long.toString(res[7]);
            result += "\nSwap free : " + Long.toString(res[8]);

            freeMem = res[2];
            usedMem = res[1];
            totalMem = res[0];

            float percent = (((float) usedMem) / ((float) totalMem)) * 100;
            return Float.toString(percent);

        } catch (ProbeException e) {
            //e.printStackTrace();
            logger.log(Level.SEVERE,"Error while getting memory probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();
            logger.log(Level.SEVERE,"Error while getting memory probe "+e);
        }
        return null;
    }

    public String getSwap() {
        String result = "No Memory result";
        long freeSwap = 0;
        long usedSwap = 0;
        long totalSwap = 0;

        try {
            long[] res = memProbe.getValue(memProbe.getResourceIds());

            result = "Total Memory : " + Long.toString(res[0]);
            result += "\nUsed Memory : " + Long.toString(res[1]);
            result += "\nFree Memory : " + Long.toString(res[2]);
            result += "\nSwap total : " + Long.toString(res[6]);
            result += "\nSwap used : " + Long.toString(res[7]);
            result += "\nSwap free : " + Long.toString(res[8]);

            freeSwap = res[8];
            usedSwap = res[7];
            totalSwap = res[6];

            float percent = (((float) usedSwap) / ((float) totalSwap)) * 100;
            return Float.toString(percent);

        } catch (ProbeException e) {
            //e.printStackTrace();
            logger.log(Level.SEVERE,"Error while getting swap memory probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();
            logger.log(Level.SEVERE,"Error while getting swap memory probe "+e);
        }
        return null;
    }


    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        if (getMemory() != null){
            return true;
        }
        return false;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }
    
}
