/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: Network.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.network;

import org.objectweb.lewys.probe.Probe;
import org.objectweb.lewys.probe.linux.NetworkProbe;
import org.objectweb.lewys.common.exceptions.NoResourceToProbeException;
import org.objectweb.lewys.common.exceptions.ProbeException;
import org.objectweb.lewys.common.exceptions.NoSuchResourceException;
import org.objectweb.jimys.monitoredSystem.pump.Pump;

import java.util.logging.Logger;
import java.util.logging.Level;

/**
* Network probe
* @author Georges Goebel, Marc Calvisi
*/
public class Network implements NetworkMBean{

    private static final String ID = "Network";

    public static final int ERROR = -1;

    private Probe networkProbe = null;
    private String[] names = null;

    private int sampleTime = 2000;

    private String device = null;

    private static Logger logger = Logger.getLogger(Network.class.getName());

    public Network() {

        try {
            networkProbe = new NetworkProbe();
            names =  networkProbe.getResourceNames();

        } catch (NoResourceToProbeException e) {
            logger.log(Level.SEVERE, "Error while creating the Network probe "+e);
        }


    }

    public long getBytesReceived(String device) {

        try {
            long[] res = networkProbe.getValue(networkProbe.getResourceIds());
            int offset = getID(device);
            return res[0+offset];
        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getBytesReceived in the Network probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getBytesReceived in the Network probe "+e);
        }

        return ERROR;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public long getPacketsReceived(String device) {
        try {
            long[] res = networkProbe.getValue(networkProbe.getResourceIds());
            int offset = getID(device);
            return res[1+offset];

        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getPacketsReceived in the Network probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getPacketsReceived in the Network probe "+e);
        }


        return ERROR;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public long getErrorsReceived(String device) {
        try {
            long[] res = networkProbe.getValue(networkProbe.getResourceIds());
            int offset = getID(device);
            return res[2+offset];

        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getErrorsReceived in the Network probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getErrorsReceived in the Network probe "+e);
        }

        return ERROR;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public long getBytesTransmit(String device) {
        try {
            long[] res = networkProbe.getValue(networkProbe.getResourceIds());
            int offset = getID(device);
            return res[8+offset];

        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getBytesTransmit in the Network probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getBytesTransmit in the Network probe "+e);
        }

        return ERROR;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public long getPacketsTransmit(String device) {
        try {
            long[] res = networkProbe.getValue(networkProbe.getResourceIds());
            int offset = getID(device);
            return res[9+offset];

        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getPacketsTransmit in the Network probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getPacketsTransmit in the Network probe "+e);
        }

        return ERROR;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public long getErrorsTransmit(String device) {
        try {
            long[] res = networkProbe.getValue(networkProbe.getResourceIds());
            int offset = getID(device);
            return res[10+offset];

        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getErrorsTransmit in the Network probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getErrorsTransmit in the Network probe "+e);
        }

        return ERROR;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public long getCollisions(String device) {
        try {
            long[] res = networkProbe.getValue(networkProbe.getResourceIds());
            int offset = getID(device);
            return res[13+offset];

        } catch (ProbeException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getCollisions in the Network probe "+e);
        } catch (NoSuchResourceException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Error while getCollisions in the Network probe "+e);
        }

        return ERROR;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public double getTxNetworkLoad(String device) {
        synchronized(this){
            long transmit1 = getBytesTransmit(device);
            long time1 = System.currentTimeMillis();
            try {
                this.wait(sampleTime);
            } catch (InterruptedException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while getTxNetworkLoad in the Network probe "+e);
            }
            long transmit2 = getBytesTransmit(device);
            long time2 = System.currentTimeMillis();

            long divTransmission = (transmit2 - transmit1);
            long divTime = (time2 - time1);

            double result = ((double)divTransmission / (double)divTime)*100;

            return result;
        }
    }

    public double getRxNetworkLoad(String device) {
        synchronized(this){
            long transmit1 = getBytesReceived(device);
            long time1 = System.currentTimeMillis();
            try {
                this.wait(sampleTime);
            } catch (InterruptedException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while getRxNetworkLoad in the Network probe "+e);
            }
            long transmit2 = getBytesReceived(device);
            long time2 = System.currentTimeMillis();

            long divTransmission = (transmit2 - transmit1);
            long divTime = (time2 - time1);

            double result = ((double)divTransmission / (double)divTime)*100;

            return result;

        }
    }

    public double getTxNetworkLoad() {
        if (device == null) {
            //System.out.println("YOU MUST INIT THE DEVICE OF THE PROBE FIRST");
            logger.log(Level.WARNING, "YOU MUST INIT THE DEVICE OF THE PROBE FIRST");
            return ERROR;
        }
        synchronized(this){
            long transmit1 = getBytesTransmit(device);
            long time1 = System.currentTimeMillis();
            try {
                this.wait(sampleTime);
            } catch (InterruptedException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while getTxNetworkLoad in the Network probe "+e);
            }
            long transmit2 = getBytesTransmit(device);
            long time2 = System.currentTimeMillis();

            long divTransmission = (transmit2 - transmit1);
            long divTime = (time2 - time1);

            double result = ((double)divTransmission / (double)divTime)*100;

            return result;
        }
    }

    public double getRxNetworkLoad() {
        if (device == null) {
            logger.log(Level.WARNING, "YOU MUST INIT THE DEVICE OF THE PROBE FIRST");
            return ERROR;
        }
        synchronized(this){
            long transmit1 = getBytesReceived(device);
            long time1 = System.currentTimeMillis();
            try {
                this.wait(sampleTime);
            } catch (InterruptedException e) {
                //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
                logger.log(Level.SEVERE, "Error while getRxNetworkLoad in the Network probe "+e);
            }
            long transmit2 = getBytesReceived(device);
            long time2 = System.currentTimeMillis();

            long divTransmission = (transmit2 - transmit1);
            long divTime = (time2 - time1);

            double result = ((double)divTransmission / (double)divTime)*100;

            return result;

        }
    }

    public void init(String device) {
        this.device = device;
    }


    public void setSampleLoadTime(int sampleTime) {
        this.sampleTime = sampleTime;
    }

    public int getSampleLoadTime() {
        return sampleTime;  //To change body of implemented methods use File | Settings | File Templates.
    }

    private int getID (String device){
        if (device == null) return -1;
        if (device.trim().toLowerCase().equals("lo")) return 0;
        if (device.trim().toLowerCase().equals("eth0")) return 1;

        int intDevice = ERROR;
        if (device.trim().toLowerCase().indexOf("eth") > -1){
            String sIntDevice = device.trim().toLowerCase().substring(device.trim().toLowerCase().indexOf("eth"), device.length());
            try {
                intDevice = Integer.parseInt(sIntDevice);
            } catch (Exception e){
                //e.printStackTrace();
                logger.log(Level.SEVERE, "Error while getID in the Network probe "+e);
            }
        }
        return intDevice;
    }

    public String getIdentifier() {
        return ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

    public boolean isAlive() {
        boolean result = false;
        if (this.device != null) {
            if (getBytesReceived(this.device) != ERROR){
                result = true;
            }
        }
        return result;
    }

    public String getFullIdentifier() {
        return Pump.getHostName()+";domain;instance;probe;"+ID;  //To change body of implemented methods use File | Settings | File Templates.
    }

}
