/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: NetworkMBean.java,v 1.2 2005/09/15 14:22:14 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.probe.network;

import org.objectweb.jimys.monitoredSystem.probe.Common;

/**
* Network probe
* @author Georges Goebel, Marc Calvisi
*/
public interface NetworkMBean extends Common{
    /**
     * Getter for BytesReceived for a network device
     * @param device name of the network device (f.ex eth0)
     * @return BytesReceived
     */
    public long getBytesReceived (String device);

    /**
     * Getter for PacketsReceived for a network device
     * @param device name of the network device (f.ex eth0)
     * @return PacketsReceived
     */
    public long getPacketsReceived(String device);

    /**
     * Getter for ErrorsReceived for a network device
     * @param device name of the network device (f.ex eth0)
     * @return ErrorsReceived
     */
    public long getErrorsReceived(String device);

    /**
     * Getter for BytesTransmit for a network device
     * @param device name of the network device (f.ex eth0)
     * @return BytesTransmit
     */
    public long getBytesTransmit(String device);

    /**
     * Getter for PacketsTransmit for a network device
     * @param device name of the network device (f.ex eth0)
     * @return PacketsTransmit
     */
    public long getPacketsTransmit(String device);

    /**
     * Getter for ErrorsTransmit for a network device
     * @param device name of the network device (f.ex eth0)
     * @return ErrorsTransmit
     */
    public long getErrorsTransmit(String device);

    /**
     * Getter for Collisions for a network device
     * @param device name of the network device (f.ex eth0)
     * @return Collisions
     */
    public long getCollisions(String device);

    /**
     * Getter for TxNetworkLoad for a network device
     * @param device name of the network device (f.ex eth0)
     * @return TxNetworkLoad
     */
    public double getTxNetworkLoad (String device);

    /**
     * Getter for RxNetworkLoad for a network device in percent
     * @param device name of the network device (f.ex eth0)
     * @return RxNetworkLoad
     */
    public double getRxNetworkLoad (String device);

    /**
     * Getter for TxNetworkLoad for a network device in percent
     * @return TxNetworkLoad
     */
    public double getTxNetworkLoad ();

    /**
     * Getter for RxNetworkLoad for a network device in percent
     * @return RxNetworkLoad
     */
    public double getRxNetworkLoad ();

    /**
     * Initialize the network probe with a network device
     * @param device Name of the network device
     */
    public void init (String device);

    /**
     * Setter for the sampletime in ms to calculate the load values
     * Default is 2 seconds
     * @param sampleTime The new sample time used to calculate the load
     */
    public void setSampleLoadTime (int sampleTime);

    /**
     * Getter for the sampletime used to calculate the load values
     * @return The sample time in ms
     */
    public int getSampleLoadTime ();


}
