/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: PumpMBean.java,v 1.3 2005/09/15 14:22:15 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.pump;

import javax.management.MalformedObjectNameException;
import javax.management.InstanceNotFoundException;
import java.util.Collection;

/**
 * The pump interface
 * @author Georges Goebel, Marc Calvisi
 */
public interface PumpMBean {
    /**
     * The time of the notification timer which returns the data to the clients
     * @return The sample time
     */
    public Long getSampleTime();

    /**
     * A new sample time for the timer which returns the data to the clients
     * @param sampleTime The new value
     */
    public void setSampleTime(Long sampleTime);

    /**
     * Get the sample rate of a probe
     * @param probename The objectname in string format
     * @return The sample rate of the pump. Null if not found
     */
    public Long getProbeSampleTime(String probename);

    /**
     * Set a new Sample rate a probe
     * @param probename The probane (ObjectName)
     * @param newSampleTime The new sample rate
     */
    public void setProbeSampleTime (String probename, Long newSampleTime);

    /**
     * Adds a new notification to the pump
     * @param mBeanName
     * @throws MalformedObjectNameException
     * @throws InstanceNotFoundException
     */
    public void addEmergencyProbeNotification(String mBeanName) throws MalformedObjectNameException, InstanceNotFoundException;

    /**
     * Starts a new probe in the pump
     * @param mBeanName The objectname of the probe
     * @param attributes The attributes of the MBean to observe
     * @param sampleTime The probe sample time
     * @throws MalformedObjectNameException
     */
    public void runMonitoringOnProbe(String mBeanName, Collection attributes, Long sampleTime) throws MalformedObjectNameException;

    /**
     * Stops all the probes (1 thread for each probe)
     */
    public void stopAllThreads ();

    /**
     * Stops one probe (one thread)
     * @param objectName The objectname of the probe
     */
    public void stopThread (String objectName);

    /**
     * Sets a new value for the maximum of the collected items
     * @param maximumCollected The new value
     */
    public void setMaximumCollected(int maximumCollected);

    /**
     * Gets the value for the maximum of collected values from the probes
     * @return The value
     */
    public int getMaximumCollected();

    /**
     * Getter for the compressing variabe
     * @return is the result compressed
     */
    public boolean isCompressing();

    /**
     * Enable or disable the compression
     * @param compressing compression
     */
    public void modifyCompressing (Boolean compressing);

    /**
     * Receive the headbeat output. The string is in xml format
     * @return hearbeat information in xml
     */
    public String getHeartBeat();

    /**
     * List of registed probes
     * @return The list of registed probes
     */
    public Collection getRegisteredProbes ();

}
