/**
* Copyright (C) 2005 - Bull S.A.
*
* Jimys - J2EE is monitoring your system
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*
* --------------------------------------------------------------------------
* $Id: StartMonitorWithConfigFile.java,v 1.3 2005/09/15 14:22:15 goebelg Exp $
* --------------------------------------------------------------------------
*/
package org.objectweb.jimys.monitoredSystem.start;

import mx4j.tools.config.ConfigurationException;
import mx4j.tools.config.ConfigurationLoader;

import javax.management.*;
import javax.management.loading.MLet;
import javax.management.loading.MLetMBean;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
* StartMonitorWithConfigFile
* @author Georges Goebel, Marc Calvisi
*/
public class StartMonitorWithConfigFile {
    private static Logger logger = Logger.getLogger(StartMonitorWithConfigFile.class.getName());

    public static void main(String[] args) {
        if (args.length != 1) {
            //System.out.println("Must specify the configuration file !");
            logger.log(Level.SEVERE, "Must specify the configuration file as program argument !");
            return;
        }

        Properties properties = new Properties();
        try {
            properties.load(new FileInputStream(new File(args[0])));

            MBeanServer server = MBeanServerFactory.createMBeanServer();
            logger.log(Level.FINE, "server.getClass() = " + server.getClass());
            ConfigurationLoader loader = new ConfigurationLoader();

            ObjectName name = ObjectName.getInstance(":service=configuration");
            server.registerMBean(loader, name);

            ObjectName adminName = ObjectName.getInstance(properties.getProperty("mlet.name"));

            MLetMBean mletService = new MLet();
            server.registerMBean(mletService, adminName);


            //URL url = new URL("file:///download/tmp/goebelg/demoProjet/mlet.xml");
            URL url = new URL(properties.getProperty("mlet.file"));
            Set loadedMBeans = (Set) server.invoke(adminName, "getMBeansFromURL",
                            new Object[]{url}, new String[]{URL.class.getName()});
            logger.log(Level.FINE, "loadedMBeans = " + loadedMBeans.size());

            adminName = ObjectName.getInstance(properties.getProperty("org.objectweb.jimys.monitoredSystem.admin.name"));

            Long pumpSampleTime = null;
            if (properties.getProperty("org.objectweb.jimys.monitoredSystem.pump.sampletime") != null) {
                pumpSampleTime = new Long(Long.parseLong(properties.getProperty("org.objectweb.jimys.monitoredSystem.pump.sampletime")));
            }

            server.invoke(adminName, "createPump", new Object[]{properties.getProperty("org.objectweb.jimys.monitoredSystem.pump.name"), pumpSampleTime}, new String[]{"java.lang.String", "java.lang.Long"});

            //looking for compression

            if (properties.getProperty("org.objectweb.jimys.monitoredSystem.pump.compress") != null) {
                Boolean compression = Boolean.valueOf(properties.getProperty("org.objectweb.jimys.monitoredSystem.pump.compress"));
                server.invoke(adminName, "modifyPumpCompression", new Object[]{compression}, new String[]{"java.lang.Boolean"});
            }


            String probes = properties.getProperty("probes");
            StringTokenizer st = new StringTokenizer(probes, ";");


            while (st.hasMoreTokens()) {
                String probe = st.nextToken();

                try {

                    String mbeanName = properties.getProperty("org.objectweb.jimys.monitoredSystem.probe." + probe + ".name");
                    String initParameters = properties.getProperty("org.objectweb.jimys.monitoredSystem.probe." + probe + ".init");
                    String attributes = properties.getProperty("org.objectweb.jimys.monitoredSystem.probe." + probe + ".attributes");
                    String sampleTime = properties.getProperty("org.objectweb.jimys.monitoredSystem.probe." + probe + ".sampleTime");

                    Long lSampleTime = Long.valueOf(sampleTime);

                    //System.out.println("Loading : " + probe + " mBeanName : " + mbeanName + " attributes : " + attributes);
                    logger.log(Level.INFO, "Loading : " + probe + " mBeanName : " + mbeanName + " attributes : " + attributes);

                    if (initParameters != null) {
                        StringTokenizer initSt = new StringTokenizer(initParameters, ";");
                        String[] sParameters = new String[initSt.countTokens()];
                        Object[] oParameters = new Object[initSt.countTokens()];
                        int counter = 0;
                        while (initSt.hasMoreTokens()) {
                            String parameter = initSt.nextToken();
                            sParameters[counter] = "java.lang.String";
                            oParameters[counter++] = parameter;
                        }

                        ObjectName temp = ObjectName.getInstance(mbeanName);
                        server.invoke(temp, "init", oParameters, sParameters);

                    }

                    if (attributes != null) {
                        StringTokenizer stAttributes = new StringTokenizer(attributes, ";");
                        Collection colAttributes = new ArrayList();
                        while (stAttributes.hasMoreTokens()) {
                            String attribute = stAttributes.nextToken();
                            colAttributes.add(attribute);
                        }

                        server.invoke(adminName, "registerProbeInPump", new Object[]{properties.getProperty("org.objectweb.jimys.monitoredSystem.pump.name"), mbeanName, colAttributes, lSampleTime}, new String[]{"java.lang.String", "java.lang.String", "java.util.Collection", "java.lang.Long"});

                    }

                } catch (Exception e) {
                    //System.err.println(" -> : Error on startup for the org.objectweb.jimys.monitoredSystem.probe : " + probe + "\n" + e.getMessage());
                    logger.log(Level.SEVERE, "Error on startup for the org.objectweb.jimys.monitoredSystem.probe : " + probe + "\n" + e);
                }


            }


            //System.out.println("Finished loading probes");
            logger.log(Level.INFO, "Finished loading probes");

            Reader reader = new BufferedReader(new FileReader(properties.getProperty("mx4j.config")));
            loader.startup(reader);
            reader.close();
            logger.log(Level.INFO, "Finished starting MBeanServer");

        } catch (IOException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (NotCompliantMBeanException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (MBeanRegistrationException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (MalformedObjectNameException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (InstanceAlreadyExistsException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (ReflectionException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (InstanceNotFoundException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (MBeanException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        } catch (ConfigurationException e) {
            //e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            logger.log(Level.SEVERE, "Exception in StartMonitorWithConfigFile : "+e);
        }

    }

}
